@@ -22,6 +22,9 @@
  * @title: GESVideoTrack
  * @short_description: A standard GESTrack for raw video
  */
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include "ges-video-track.h"
 #include "ges-smart-video-mixer.h"
@@ -31,9 +34,7 @@ struct _GESVideoTrackPrivate
   gpointer nothing;
 };
 
-#define GES_VIDEO_TRACK_GET_PRIVATE(o)  (G_TYPE_INSTANCE_GET_PRIVATE ((o), GES_TYPE_VIDEO_TRACK, GESVideoTrackPrivate))
-
-G_DEFINE_TYPE (GESVideoTrack, ges_video_track, GES_TYPE_TRACK);
+G_DEFINE_TYPE_WITH_PRIVATE (GESVideoTrack, ges_video_track, GES_TYPE_TRACK);
 
 static void
 _sync_capsfilter_with_track (GESTrack * track, GstElement * capsfilter)
@@ -125,8 +126,6 @@ ges_video_track_class_init (GESVideoTrackClass * klass)
 /*   GESTrackClass *parent_class = GES_TRACK_CLASS (klass);
  */
 
-  g_type_class_add_private (klass, sizeof (GESVideoTrackPrivate));
-
   object_class->finalize = ges_video_track_finalize;
 
   GES_TRACK_CLASS (klass)->get_mixing_element = ges_smart_mixer_new;
