@@ -20,8 +20,12 @@
 
 /**
  * SECTION:gesvideourisource
+ * @title: GESVideoUriSource
  * @short_description: outputs a single video stream from a given file
  */
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include <gst/pbutils/missing-plugins.h>
 
@@ -34,7 +38,7 @@
 
 struct _GESVideoUriSourcePrivate
 {
-  void *nothing;
+  GstElement *decodebin;        /* Reference owned by parent class */
 };
 
 enum
@@ -43,6 +47,26 @@ enum
   PROP_URI
 };
 
+static void
+ges_video_uri_source_track_set_cb (GESVideoUriSource * self,
+    GParamSpec * arg G_GNUC_UNUSED, gpointer nothing)
+{
+  GESTrack *track;
+  const GstCaps *caps = NULL;
+
+  if (!self->priv->decodebin)
+    return;
+
+  track = ges_track_element_get_track (GES_TRACK_ELEMENT (self));
+  if (!track)
+    return;
+
+  caps = ges_track_get_caps (track);
+
+  GST_INFO_OBJECT (self, "Setting caps to: %" GST_PTR_FORMAT, caps);
+  g_object_set (self->priv->decodebin, "caps", caps, NULL);
+}
+
 /* GESSource VMethod */
 static GstElement *
 ges_video_uri_source_create_source (GESTrackElement * trksrc)
@@ -58,7 +82,8 @@ ges_video_uri_source_create_source (GESTrackElement * trksrc)
   if (track)
     caps = ges_track_get_caps (track);
 
-  decodebin = gst_element_factory_make ("uridecodebin", NULL);
+  decodebin = self->priv->decodebin = gst_element_factory_make ("uridecodebin",
+      NULL);
 
   g_object_set (decodebin, "caps", caps,
       "expose-all-streams", FALSE, "uri", self->uri, NULL);
@@ -97,7 +122,7 @@ ges_extractable_interface_init (GESExtractableInterface * iface)
 }
 
 G_DEFINE_TYPE_WITH_CODE (GESVideoUriSource, ges_video_uri_source,
-    GES_TYPE_VIDEO_SOURCE,
+    GES_TYPE_VIDEO_SOURCE, G_ADD_PRIVATE (GESVideoUriSource)
     G_IMPLEMENT_INTERFACE (GES_TYPE_EXTRACTABLE,
         ges_extractable_interface_init));
 
@@ -155,8 +180,6 @@ ges_video_uri_source_class_init (GESVideoUriSourceClass * klass)
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
   GESVideoSourceClass *source_class = GES_VIDEO_SOURCE_CLASS (klass);
 
-  g_type_class_add_private (klass, sizeof (GESVideoUriSourcePrivate));
-
   object_class->get_property = ges_video_uri_source_get_property;
   object_class->set_property = ges_video_uri_source_set_property;
   object_class->dispose = ges_video_uri_source_dispose;
@@ -176,8 +199,10 @@ ges_video_uri_source_class_init (GESVideoUriSourceClass * klass)
 static void
 ges_video_uri_source_init (GESVideoUriSource * self)
 {
-  self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self,
-      GES_TYPE_VIDEO_URI_SOURCE, GESVideoUriSourcePrivate);
+  self->priv = ges_video_uri_source_get_instance_private (self);
+
+  g_signal_connect (self, "notify::track",
+      G_CALLBACK (ges_video_uri_source_track_set_cb), NULL);
 }
 
 /**
@@ -186,8 +211,8 @@ ges_video_uri_source_init (GESVideoUriSource * self)
  *
  * Creates a new #GESVideoUriSource for the provided @uri.
  *
- * Returns: The newly created #GESVideoUriSource, or %NULL if there was an
- * error.
+ * Returns: (transfer floating) (nullable): The newly created #GESVideoUriSource,
+ * or %NULL if there was an error.
  */
 GESVideoUriSource *
 ges_video_uri_source_new (gchar * uri)
